@@ -39,14 +39,15 @@
 #include "calls-message-source.h"
 #include "calls-new-call-box.h"
 #include "calls-notifier.h"
+#include "calls-plugin-manager.h"
 #include "calls-record-store.h"
 #include "calls-ringer.h"
 #include "version.h"
 
+#include <adwaita.h>
 #include <call-ui.h>
 #include <glib/gi18n.h>
 #include <glib-unix.h>
-#include <handy.h>
 #include <libcallaudio.h>
 
 /**
@@ -56,28 +57,34 @@
  * @include: "calls-application.h"
  */
 
-#define DEFAULT_PROVIDER_PLUGIN "mm"
-
 struct _CallsApplication {
-  GtkApplication    parent_instance;
+  AdwApplication      parent_instance;
 
-  gboolean          daemon;
-  CallsRinger      *ringer;
-  CallsNotifier    *notifier;
-  CallsRecordStore *record_store;
-  CallsMainWindow  *main_window;
-  CallsCallWindow  *call_window;
-  CallsDBusManager *dbus_manager;
+  CallsRinger        *ringer;
+  CallsNotifier      *notifier;
+  CallsRecordStore   *record_store;
+  CallsMainWindow    *main_window;
+  CallsCallWindow    *call_window;
+  CallsDBusManager   *dbus_manager;
+  CallsManager       *manager;
+  CallsPluginManager *plugin_manager;
+  CallsSettings      *settings;
 
-  char             *uri;
-  guint             id_sigterm;
-  guint             id_sigint;
-  gboolean          shutdown;
-  gboolean          db_done;
+  char               *uri;
+  guint               id_sigterm;
+  guint               id_sigint;
+  gboolean            shutdown;
+  gboolean            db_done;
 };
 
-G_DEFINE_TYPE (CallsApplication, calls_application, GTK_TYPE_APPLICATION);
+G_DEFINE_TYPE (CallsApplication, calls_application, ADW_TYPE_APPLICATION);
 
+enum {
+  SIGNAL_MAIN_WINDOW_CLOSED,
+  N_SIGNALS
+};
+
+static guint signals[N_SIGNALS];
 
 static void start_proper (CallsApplication *self);
 
@@ -90,8 +97,14 @@ quit_calls (CallsApplication *self)
   if (self->shutdown)
     return;
 
-  gtk_application_remove_window (GTK_APPLICATION (self), GTK_WINDOW (self->main_window));
-  gtk_application_remove_window (GTK_APPLICATION (self), GTK_WINDOW (self->call_window));
+  if (self->main_window)
+    gtk_application_remove_window (GTK_APPLICATION (self), GTK_WINDOW (self->main_window));
+
+  if (self->call_window)
+    gtk_application_remove_window (GTK_APPLICATION (self), GTK_WINDOW (self->call_window));
+
+  if (g_application_get_flags (G_APPLICATION (self)) & G_APPLICATION_IS_SERVICE)
+    g_application_release (G_APPLICATION (self));
 
   self->shutdown = TRUE;
 }
@@ -156,79 +169,80 @@ calls_application_dbus_unregister (GApplication    *application,
 
 
 static void
-set_provider_names_action (GSimpleAction *action,
-                           GVariant      *parameter,
-                           gpointer       user_data)
+set_plugin_names_action (GSimpleAction *action,
+                         GVariant      *parameter,
+                         gpointer       user_data)
 {
-  CallsManager *manager;
   g_autofree const char **names = NULL;
   g_autofree const char **loaded = NULL;
+  CallsApplication *self = user_data;
   gsize length;
   guint length_loaded;
 
+  g_assert (CALLS_IS_APPLICATION (self));
+
   names = g_variant_get_strv (parameter, &length);
   g_return_if_fail (names && *names);
 
-  manager = calls_manager_get_default ();
-  loaded = calls_manager_get_provider_names (manager, &length_loaded);
+  loaded = calls_plugin_manager_get_plugin_names (self->plugin_manager, &length_loaded);
 
-  /* remove unwanted providers */
+  /* remove unwanted plugins */
   for (guint i = 0; i < length_loaded; i++) {
-    g_autofree char *provider = g_strdup (loaded[i]);
-    if (!g_strv_contains (names, provider))
-      calls_manager_remove_provider (manager, provider);
+    g_autofree char *plugin = g_strdup (loaded[i]);
+
+    if (!g_strv_contains (names, plugin)) {
+      g_autoptr (GError) error = NULL;
+      gboolean ok = calls_plugin_manager_unload_plugin (self->plugin_manager, plugin, &error);
+
+      g_debug ("Unloading plugin `%s' %ssuccessful", plugin, ok ? "" : "un");
+      if (!ok)
+        g_warning ("Plugin '%s' not unloaded: %s", plugin, error->message);
+    }
   }
 
   for (guint i = 0; i < length; i++) {
+    g_autoptr (GError) error = NULL;
     const char *name = names[i];
+    gboolean ok;
 
-    if (calls_manager_has_provider (manager, name))
+    if (calls_plugin_manager_has_plugin (self->plugin_manager, name))
       continue;
 
-    g_debug ("Loading provider `%s'", name);
-    calls_manager_add_provider (manager, name);
+    ok = calls_plugin_manager_load_plugin (self->plugin_manager, name, &error);
+    g_debug ("Loading plugin `%s' %ssuccessful", name, ok ? "" : "un");
+    if (!ok)
+      g_warning ("Plugin '%s' not loaded: %s", name, error->message);
   }
 }
 
 
 static void
-set_default_providers_action (GSimpleAction *action,
-                              GVariant      *parameter,
-                              gpointer       user_data)
+set_default_plugins_action (GSimpleAction *action,
+                            GVariant      *parameter,
+                            gpointer       user_data)
 {
-  CallsManager *manager = calls_manager_get_default ();
-  CallsSettings *settings = calls_manager_get_settings (manager);
+  CallsApplication *self = CALLS_APPLICATION (user_data);
 
   g_auto (GStrv) plugins = NULL;
   /**
    * Only add default providers when there are none added yet,
    * This makes sure we're not resetting explicitly set providers
    */
-  if (calls_manager_has_any_provider (manager))
+  if (calls_plugin_manager_has_any_plugins (self->plugin_manager))
     return;
 
-  plugins = calls_settings_get_autoload_plugins (settings);
+  plugins = calls_settings_get_autoload_plugins (self->settings);
   for (guint i = 0; plugins[i] != NULL; i++) {
-    calls_manager_add_provider (manager, plugins[i]);
+    g_autoptr (GError) error = NULL;
+
+    g_debug ("Loading default provider %s", plugins[i]);
+
+    if (!calls_plugin_manager_load_plugin (self->plugin_manager, plugins[i], &error))
+      g_warning ("Could not load plugin '%s': %s", plugins[i], error->message);
   }
 }
 
 
-static void
-set_daemon_action (GSimpleAction *action,
-                   GVariant      *parameter,
-                   gpointer       user_data)
-{
-  CallsApplication *self = CALLS_APPLICATION (user_data);
-  gboolean daemon = g_variant_get_boolean (parameter);
-
-  self->daemon = daemon;
-
-  g_debug ("Application %smarked as daemon",
-           daemon ? "" : "not ");
-}
-
-
 #define DIALLING    "0-9*#+ABCD"
 #define SIGNALLING  ",TP!W@X"
 #define VISUAL      "[:space:]\\-.()t/"
@@ -386,10 +400,10 @@ copy_number (GSimpleAction *action,
              gpointer       user_data)
 {
   const char *number = g_variant_get_string (parameter, NULL);
-  GtkClipboard *clipboard =
-    gtk_clipboard_get_default (gdk_display_get_default ());
+  GdkClipboard *clipboard =
+    gdk_display_get_clipboard (gdk_display_get_default ());
 
-  gtk_clipboard_set_text (clipboard, number, -1);
+  gdk_clipboard_set_text (clipboard, number);
 
   g_debug ("Copied `%s' to clipboard", number);
 }
@@ -418,9 +432,8 @@ manager_state_changed_cb (GApplication *application)
 
 static const GActionEntry actions[] =
 {
-  { "set-provider-names", set_provider_names_action, "as" },
-  { "set-default-providers", set_default_providers_action, NULL },
-  { "set-daemon", set_daemon_action, "b" },
+  { "set-plugin-names", set_plugin_names_action, "as" },
+  { "set-default-plugins", set_default_plugins_action, NULL },
   { "dial", dial_action, "s" },
   { "copy-number", copy_number, "s" },
   /* TODO About dialog { "about", show_about, NULL}, */
@@ -454,21 +467,11 @@ calls_application_handle_local_options (GApplication *application,
 static void
 startup (GApplication *application)
 {
-  g_autoptr (GtkCssProvider) provider = NULL;
   g_autoptr (GError) error = NULL;
-#if HDY_CHECK_VERSION (1, 5, 0)
-  HdyStyleManager *style_manager;
-#endif
 
   G_APPLICATION_CLASS (calls_application_parent_class)->startup (application);
 
-  hdy_init ();
-
-#if HDY_CHECK_VERSION (1, 5, 0)
-  style_manager = hdy_style_manager_get_default ();
-
-  hdy_style_manager_set_color_scheme (style_manager, HDY_COLOR_SCHEME_PREFER_LIGHT);
-#endif
+  g_print ("Calls %s starting up...\n", VCS_TAG);
 
   if (!call_audio_init (&error))
     g_warning ("Failed to init libcallaudio: %s", error->message);
@@ -481,6 +484,9 @@ startup (GApplication *application)
   if (!g_get_application_name ())
     g_set_application_name (_("Calls"));
 
+  if (!gtk_window_get_default_icon_name ())
+    gtk_window_set_default_icon_name (APP_ID);
+
   g_action_map_add_action_entries (G_ACTION_MAP (application),
                                    actions,
                                    G_N_ELEMENTS (actions),
@@ -492,13 +498,12 @@ startup (GApplication *application)
                            application,
                            G_CONNECT_SWAPPED);
 
-  manager_state_changed_cb (application);
+  if (g_application_get_flags (application) & G_APPLICATION_IS_SERVICE) {
+    g_application_hold (application);
+    g_debug ("Enable daemon mode");
+  }
 
-  provider = gtk_css_provider_new ();
-  gtk_css_provider_load_from_resource (provider, "/org/gnome/Calls/style.css");
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  manager_state_changed_cb (application);
 }
 
 
@@ -510,32 +515,13 @@ calls_application_command_line (GApplication            *application,
   GVariantDict *options;
   const char *arg;
 
-  g_autoptr (GVariant) providers = NULL;
+  g_autoptr (GVariant) plugins = NULL;
   g_auto (GStrv) arguments = NULL;
   gint argc;
   guint verbosity;
 
   options = g_application_command_line_get_options_dict (command_line);
 
-  providers = g_variant_dict_lookup_value (options, "provider", G_VARIANT_TYPE_STRING_ARRAY);
-  if (providers) {
-    g_action_group_activate_action (G_ACTION_GROUP (application),
-                                    "set-provider-names",
-                                    providers);
-  } else {
-    g_action_group_activate_action (G_ACTION_GROUP (application),
-                                    "set-default-providers",
-                                    NULL);
-  }
-
-  g_action_group_activate_action (G_ACTION_GROUP (application),
-                                  "set-daemon",
-                                  g_variant_new_boolean (g_variant_dict_contains (options, "daemon")));
-
-  if (g_variant_dict_lookup (options, "dial", "&s", &arg))
-    g_action_group_activate_action (G_ACTION_GROUP (application),
-                                    "dial", g_variant_new_string (arg));
-
   /* TODO make this a comma separated string of "CATEGORY:level" pairs */
   if (g_variant_dict_lookup (options, "verbosity", "u", &verbosity)) {
     gint delta = calls_log_set_verbosity (verbosity);
@@ -547,6 +533,22 @@ calls_application_command_line (GApplication            *application,
                level);
   }
 
+  start_proper (self);
+
+  plugins = g_variant_dict_lookup_value (options, "plugins", G_VARIANT_TYPE_STRING_ARRAY);
+  if (plugins) {
+    g_action_group_activate_action (G_ACTION_GROUP (application),
+                                    "set-plugin-names",
+                                    plugins);
+  } else {
+    g_action_group_activate_action (G_ACTION_GROUP (application),
+                                    "set-default-plugins",
+                                    NULL);
+  }
+
+  if (g_variant_dict_lookup (options, "dial", "&s", &arg))
+    g_action_group_activate_action (G_ACTION_GROUP (application),
+                                    "dial", g_variant_new_string (arg));
   arguments = g_application_command_line_get_arguments (command_line, &argc);
 
   /* Keep only the first URI, if there are many */
@@ -617,6 +619,16 @@ on_db_done (CallsRecordStore *store,
     g_warning ("Database did not get opened");
 }
 
+static gboolean
+on_main_window_hidden (GtkWidget *widget,
+                       gpointer data)
+{
+  CallsApplication *self = CALLS_APPLICATION (g_application_get_default ());
+
+  g_signal_emit_by_name (self, "main-window-closed", 0);
+
+  return FALSE;
+}
 
 static void
 start_proper (CallsApplication *self)
@@ -629,6 +641,15 @@ start_proper (CallsApplication *self)
 
   gtk_app = GTK_APPLICATION (self);
 
+  self->settings = calls_settings_get_default ();
+  g_assert (self->settings);
+
+  self->plugin_manager = calls_plugin_manager_get_default ();
+  g_assert (self->plugin_manager);
+
+  self->manager = calls_manager_get_default ();
+  g_assert (self->manager);
+
   self->ringer = calls_ringer_new ();
   g_assert (self->ringer != NULL);
 
@@ -644,7 +665,7 @@ start_proper (CallsApplication *self)
 
   self->main_window =
     calls_main_window_new (gtk_app,
-                           G_LIST_MODEL (self->record_store));
+                           calls_record_store_get_list_model (self->record_store));
   g_assert (self->main_window != NULL);
 
   self->call_window = calls_call_window_new (gtk_app);
@@ -655,23 +676,23 @@ start_proper (CallsApplication *self)
                            G_CALLBACK (notify_window_visible_cb),
                            self,
                            G_CONNECT_AFTER);
-}
 
+  g_signal_connect (G_OBJECT (self->main_window),
+                    "hide",
+                    G_CALLBACK (on_main_window_hidden),
+                    NULL);
+}
 
 static void
 activate (GApplication *application)
 {
   CallsApplication *self = CALLS_APPLICATION (application);
-  gboolean present;
 
   g_debug ("Activated");
 
   start_proper (self);
-  present = !self->daemon;
 
-  if (present || self->uri) {
-    gtk_window_present (GTK_WINDOW (self->main_window));
-  }
+  gtk_window_present (GTK_WINDOW (self->main_window));
 
   if (self->uri) {
     if (g_str_has_prefix (self->uri, "tel:"))
@@ -735,13 +756,17 @@ finalize (GObject *object)
   g_clear_handle_id (&self->id_sigint, g_source_remove);
 
   if (self->main_window)
-    gtk_widget_destroy (GTK_WIDGET (self->main_window));
+    gtk_window_destroy (GTK_WINDOW (self->main_window));
   if (self->call_window)
-    gtk_widget_destroy (GTK_WIDGET (self->call_window));
+    gtk_window_destroy (GTK_WINDOW (self->call_window));
 
   g_clear_object (&self->record_store);
   g_clear_object (&self->ringer);
   g_clear_object (&self->notifier);
+  g_clear_object (&self->manager);
+  g_clear_object (&self->plugin_manager);
+  g_clear_object (&self->settings);
+
   g_free (self->uri);
 
   G_OBJECT_CLASS (calls_application_parent_class)->finalize (object);
@@ -754,6 +779,13 @@ calls_application_class_init (CallsApplicationClass *klass)
   GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  signals [SIGNAL_MAIN_WINDOW_CLOSED] =
+    g_signal_new ("main-window-closed",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL, NULL,
+                  G_TYPE_NONE, 0);
+
   object_class->finalize = finalize;
 
   application_class->handle_local_options = calls_application_handle_local_options;
@@ -775,17 +807,11 @@ calls_application_init (CallsApplication *self)
 {
   const GOptionEntry options[] = {
     {
-      "provider", 'p', G_OPTION_FLAG_NONE,
+      "plugins", 'p', G_OPTION_FLAG_NONE,
       G_OPTION_ARG_STRING_ARRAY, NULL,
-      _("The name of the plugin to use as a call provider"),
+      _("The name of the plugins to load"),
       _("PLUGIN")
     },
-    {
-      "daemon", 'd', G_OPTION_FLAG_NONE,
-      G_OPTION_ARG_NONE, NULL,
-      _("Whether to present the main window on startup"),
-      NULL
-    },
     {
       "dial", 'l', G_OPTION_FLAG_NONE,
       G_OPTION_ARG_STRING, NULL,
